import json
from . import local_config
from . import request

def create_workflow(workflowname, workflowcode, step, preStep):
    if local_config.check():
        data = {
            "workflowName" : workflowname,
            "workflowCode" : workflowcode,
            "step" : step,
            "preStep" : json.dumps(list(preStep))
        };
        response = request.post("workflow", "create", json.dumps(data))
        if not response:
            return
        if response["code"] == 0:
            print("[Biolab] Workflow " + workflowname + "#" + workflowcode + "_" + step + " registered successful !")
        else:
            print("[Biolab] Workflow " + workflowname + "#" + workflowcode + "_" + step + " already registered")



def list_workflow(workflowcode):
    if local_config.check():
        data = [workflowcode]
        if not workflowcode:
            data = []
        # print(list(unique_no))
        response = request.post("workflow", "list", json.dumps(data))
        if not response:
            return
        if not response['data']:
            print("No workflow configured")
            return
        for workflow in response["data"]:
            print(workflow)


import click

from . import local_config
from . import bio_data
from . import pipeline
from . import bio_script
from . import bio_env
from . import bio_task
from . import server_config

@click.group()
def cli():
    """A simple biolab command client, See https://github.com/BioHubX/biocmd  for more detail information"""
    pass

@cli.group()
def workflow():
    """ Define the dependencies between processes

    For example: biocmd workflow create --name 'Strain Analysis pipeline' --code StrainAnalysis --step assembly --prestep qcstat --prestep datafilter

    The above example means that creating a new pipeline requires specifying the name of the pipeline, the unique code of the pipeline, the name of the current algorithm of the pipeline, and the name of the pre-algorithm ["qcstat","datafilter"] that the algorithm depends on
    """
    pass

@cli.group()
def script():
    """ Configure the algorithm used in the analysis flow

    For example: biocmd script create --workflow StrainAnalysis --step qcstat --script biohubx/qcstat:v0.1.0

    The above example means that creating an analysis algorithm requires specifying the name of a workflow, the name of the algorithm, and the algorithm implementation of the docker image corresponding to the algorithm
    """
    pass

@cli.group()
def env():
    """  Configure the Input,Output and Environment parameters of the algorithm

    For example: biocmd env create --workflow StrainAnalysis --step qcstat --type Input --key file1 --feature _R1.fq.gz

    For the above examples, that means that for the qcstat analysis algorithm in the StrainAnalysis workflow,a dependency on an input parameter is required, with the parameter name being file1, and the feature of the value of this parameter being that it contains _R1.fq.gz.
    """
    pass

@cli.group()
def file():
    """ Data file management,such as data registration

    For example: biocmd file create --basedir /workspace/20241205

    The above example means that the "/workspace/20241205" folder is used as the starting point to scan the address of the original sequencing file, which should meet certain format requirements, such as: /{baseDir}/{sampleNo}/{uniqueNo}/{sampleNo.R1.fq.gz},the final file structure looks like this:
        sample1 => /workspace/20241205/sample1/unique1/sample1_R1.fq.gz
        sample1 => /workspace/20241205/sample1/unique1/sample1_R2.fq.gz

        sample2 => /workspace/20241205/sample2/unique2/sample2_R1.fq.gz
        sample2 => /workspace/20241205/sample2/unique2/sample2_R2.fq.gz

        and more ...

    sampleNo can same as uniqueNo
    """
    pass

@cli.group()
def task():
    """ Task management, such as starting a task and viewing a task

    For example: biocmd task create --workflow StrainAnalysis --step qcstat --uniqueno sample1

    The above example means that an StrainAnalysis_qcstat analysis task has been created for the sample1 (unique sample number)
    """
    pass

@cli.group()
def server():
    """ Configure the remote server environment, only support: server,parallel,outputDir

    For example: biocmd server --key parallel --value 30

    The biolab server specifies that the maximum concurrency capacity of the remote docker machine is 30 when doing automated scheduling (compute nodes can only allow a maximum of 30 tasks to be performed simultaneously).
    """
    pass

@cli.group()
def local():
    """ Configure the remote server address, port, and token

    For example: biocmd local set --server http://localhost --port 8080 --token xxxxxxxx

    The above example means that the client is connected to the biolab server deployed at localhost, the service exposed port is 8080, and the token is a string automatically generated when the biolab service is started
    """
    pass


@local.command()
@click.option('--server', required=True, default='http://localhost', help='server http address, eg: http://biolab.com')
@click.option('--port', required=True, default='8080', help='server port, eg 8080')
@click.option('--token', required=True, default='', help='Token of the biolab server')
def set(server,port,token):
    """
        --server,--port, --port optional be provided
    """
    local_config.set_config(server, port, token)


@workflow.command()
@click.option('--name', required=True, help='Pipeline name')
@click.option('--code', required=True, help='Pipeline code')
@click.option('--step', required=True, help='Step name in the pipeline code')
@click.option('--prestep', multiple=True, required=False, help='The pre-step of the current step in the pipeline')
def create(name,code,step, prestep):
    """
        --name, --code, --step, --prestep must be provided
    """
    for pre in prestep:
        if not pre.strip():
            print("Prestep value contains empty string")
            return
    pipeline.create_workflow(name, code, step, prestep)


@workflow.command()
@click.option('--workflow', required=False, help='Pipeline code')
def list(workflow):
    """
        --workflow must be provided
    """
    pipeline.list_workflow(workflow)


@script.command()
@click.option('--workflow', required=True, help='Pipeline code')
@click.option('--step', required=True, help='Step name in the pipeline code')
@click.option('--script', required=True, help='Docker image name (include version)')
def create(workflow,step, script):
    """
        --workflow, --step, --script must be provided
    """
    bio_script.create_script(workflow,step, "Docker", script)

@script.command()
@click.option('--workflow', required=False, help='-')
def list(workflow):
    """
        --workflow must be provided
    """
    bio_script.list_script(workflow)


@env.command()
@click.option('--workflow', required=True, help='Unique pipeline code')
@click.option('--step', required=True, help='Step name in a pipeline')
@click.option('--type', required=True,  help='Only the one of [Input,Output,Mount]')
@click.option('--key', required=True, help='Feature unique key')
@click.option('--feature', required=True, help='qcstat:v1.0.0')
def create(workflow,step,type, key, feature):
    """
        --workflow,--step, --type, --key, --feature must be provided
    """
    bio_env.create_env(workflow,step, type, key, feature)

@env.command()
@click.option('--workflow', required=True, help='Unique workflow code')
@click.option('--step', required=True, help='Step name in a pipeline')
def list(workflow, step):
    """
        --workflow,--step must be provided
    """
    bio_env.list_env(workflow, step)

@file.command()
@click.option('--basedir', required=True, help="Base dir of some sample's rawdata")
def create(basedir):
    """
        --basedir must be provided
    """
    bio_data.create_file(basedir)


@file.command()
@click.option('--uniqueno', multiple=True, required=False, help='Unique name of a sample')
def list(uniqueno):
    """
       --uniqueno optional be provided
    """
    bio_data.list_file(uniqueno)

@task.command()
@click.option('--workflow', required=True, help='Unique pipeline code')
@click.option('--step', required=True, help='Step name in a pipeline')
@click.option('--uniqueno', required=True, help='Unique name of a sample')
def create(workflow,step,uniqueno):
    """
        --workflow,--step, --uniqueno must be provided
    """
    bio_task.create_task(workflow,step, uniqueno)

# @task.command()
# @click.option('--workflow', required=True, help='Unique pipeline code')
# def list(workflow):
#     """
#         --workflow  must be provided
#     """
#     bio_task.list_task(workflow)

@server.command()
@click.option('--key', required=True, help='Only one of [server,parallel,outputDir]')
@click.option('--value', required=True, help='The value of the key')
def create(key,value):
    """
        --key,--value must be provided
    """
    server_config.create_config("Docker", key, value)


@server.command()
def list():
    """
        list all of the server config
    """
    server_config.list_config()

if __name__ == '__main__':
    cli()



import json
from . import local_config
from . import request

def create_script(workflowcode, step, type, script):
    if local_config.check():
        data = {
            "workflowCode" : workflowcode,
            "step" : step,
            "type" : type,
            "script" : script
        }
        response = request.post("script", "create", json.dumps(data))
        if not response:
            return
        if response["code"] == 0:
            print("[Biolab] Script " + workflowcode + "_" + step + "#"  + script  + " registered successful")
        else:
            print("[Biolab] Script " + workflowcode + "_" + step + "#"  + script  + " already registered")


def list_script(workflowcode):
    if local_config.check():
        data = [workflowcode];
        if not workflowcode:
            data = []
        response = request.post("script", "list", json.dumps(data))
        if not response:
            return
        if not response['data']:
            print("No scripts configured")
            return
        for script in response['data']:
            print(script)


import json
from . import local_config
from . import request

def create_config(type, key, value):
    if local_config.check():
        data = {
            "type" : type,
            "key" : key,
            "value" : value
        };
        if key not in ['server','parallel','outputDir']:
            print("[Biolab] The key only support ['server','parallel','outputDir'] currently")
            return
        response = request.post("config", "create", json.dumps(data))
        if not response:
            return
        if response['code'] == 0:
            print("[Biolab] The config " + type + "_" + key + " registered successful")
        else:
            print("[Biolab] The config " + type + "_" + key + " already registered")


def list_config():
    if local_config.check():
        data = {};
        response = request.post("config", "list", json.dumps(data))
        if not response:
            return
        if not response['data']:
            print("No system configs configured")
            return
        for config in response['data']:
            print(config)


import requests
import os
import sys
from . import local_config

route_dict = {
  "fileIndex": {
    "create": "/index/create",
    "createAll": "/index/createAll",
    "list": "/index/list"
  },
  "workflow": {
    "create": "/workflow/create",
    "list": "/workflow/list"
  },
  "script": {
    "create": "/script/create",
    "list": "/script/list"
  },
  "env": {
    "create": "/env/create",
    "list": "/env/list"
  },
  "task": {
    "create": "/task/create",
    "list": "/task/list"
  },
  "config": {
    "create": "/config/create",
    "list": "/config/list"
  }
}

sys.tracebacklimit = 0
def post(module, method, data):
    config_dict = local_config.load_config()
    url = config_dict["server"] + ":" + config_dict["port"];
    headers = {
        'Content-Type': 'application/json',
        'biolab-token': config_dict["token"]
    }
    url += route_dict[module][method]
    # print("[Biolab] ", url, data)
    try:
        response = requests.request("POST", url, data=data, headers=headers)
        # print(response.json())
        resp = response.json()
        if(resp['code']) == 500:
            print("Sorry, server has an unknown exception in currently")
            resp['data'] = [""]
        return resp
    except Exception as e:
        print(e)
        home_dir = os.path.expanduser('~')
        config_path = os.path.join(home_dir, '.biocmd.conf');
        print("Can't communicate with server, maybe token is invalid, please check your local config " + config_path)




import os
import json

def set_config(server,port,token):
    config = {
        "server": server,
        "port": port,
        "token": token
    }
    home_dir = os.path.expanduser('~')
    config_path = os.path.join(home_dir, '.biocmd.conf');

    print("[Biolab] Config write in [", config_path, "] successful!")    # 配置服务端信息
    with open(config_path, 'w') as file:
        file.write(json.dumps(config,indent=4));

def load_config():
    home_dir = os.path.expanduser('~')
    config_path = os.path.join(home_dir, '.biocmd.conf');
    if os.path.exists(config_path):
        with open(config_path, 'r') as f:
            config_string = f.read()
            return json.loads(config_string)
    else:
        print("[Biolab] Use 'biocmd local' command to set 'server','port' and 'token' first");
        return {}

def check():
    config = load_config();
    if len(config) == 0:
        return False;
    if "server" not in config or "port" not in config:
        print("[Biolab] 'server','port' and 'token' must be configured, use config command first")
        return False;
    return True;


import json
from . import local_config
from . import request

def create_task(workflowcode, step, uniqueNo):
    if local_config.check():
        data = {
            "workflowCode" : workflowcode,
            "step" : step,
            "uniqueNo" : uniqueNo
        }
        response = request.post("task", "create", json.dumps(data))
        if not response:
            return
        code = response["code"]
        if code == 0:
            print("[Biolab] Task " + workflowcode + "_" + step + "_"  + uniqueNo  + " create successful")
        elif code == 5003:
            print("[Biolab] The workflow [" + workflowcode + "_" + step + "] missing script")
        elif code == 5004:
            print("[Biolab] The workflow [" + workflowcode + "_" + step + "] missing input template")
        elif code == 5005:
            print("[Biolab] The workflow [" + workflowcode + "_" + step + "] missing input files")
        elif code == 5006:
            print("[Biolab] The task [" + workflowcode + "_" + step + "_" + uniqueNo + "] pre task not all completed")
        elif code == 5007:
            print("[Biolab] The task [" + workflowcode + "_" + step + "_" + uniqueNo + "] already exist")
        elif code == 5008:
            print("[Biolab] The workflow [" + workflowcode + "_" + step + "] not exist")

def list_task(workflowcode):
    if local_config.check():
        data = [workflowcode]
        if not workflowcode:
            data = []
        response = request.post("task", "list", json.dumps(data))
        if not response:
            return
        if not response['data']:
            print("No tasks created")
            return
        for script in response['data']:
            print(script)


from setuptools import setup, find_packages

setup(
    name='biocmd',
    version='0.1.1',
    description='BioLab Client',
    author='Yaping-Liu',
    author_email='applesline@163.com',
    license='Apache License 2.0',
    url="https://github.com/BioHubX/biocmd",
    packages = ['biocmd'],
    install_requires=[
        'click',
    ],
    entry_points={
        'console_scripts': [
            'biocmd=biocmd.entry:cli',
        ],

    },
)


import json
from . import local_config
from . import request

def create_env(workflowcode, step, type, key, feature):
    if local_config.check():
        data = {
            "workflowCode" : workflowcode,
            "step" : step,
            "type" : type,
            "key" : key,
            "feature" : feature
        };
        response = request.post("env", "create", json.dumps(data))
        if not response:
            return
        if response['code'] == 0:
            print("[Biolab] The env " + workflowcode + "_" + step + "_" + type + "#" + key + " registered successful")
        else:
            print("[Biolab] The env " + workflowcode + "_" + step + "_" + type + "#" + key + " already registered")


def list_env(workflowcode, step):
    if local_config.check():
        data = {
            "workflowCode" : workflowcode,
            "step" : step
        };
        response = request.post("env", "list", json.dumps(data))
        if not response:
            return
        if not response['data']:
            print("No envs added")
            return
        for env in response['data']:
            print(env)


import os
import json

from . import local_config
from . import request

def create_file(basedir):
    if local_config.check():
        if basedir.endswith('/'):
             basedir = basedir[:-1]
        batchNo = os.path.basename(basedir)
        data = [];
        file_address = [];
        for root, dirs, files in os.walk(basedir):
            for file in files:
                if file.endswith('.fq.gz'):
                    sample_no, unique_no = os.path.basename(os.path.dirname(root)), os.path.basename(root)
                    file_path = os.path.join(root, file)
                    found = False
                    for item in data:
                        if item['sampleNo'] == sample_no and item['uniqueNo'] == unique_no:
                            item['address'].append(file_path)
                            file_address = item['address']
                            item['address'] = json.dumps(file_address)
                            found = True
                            break
                    if not found:
                        data.append({'batchNo':batchNo,'sampleNo': sample_no, 'uniqueNo': unique_no,'type':'rawdata', 'address': [file_path]})

        # print(data)
        response = request.post("fileIndex", "createAll", json.dumps(data))
        if not response:
            return
        if response['code'] == 0:
            print("[Biolab] Total", len(data) , "samples register successful")


def list_file(uniqueno):
    if local_config.check():
        response = request.post("fileIndex", "list", json.dumps(uniqueno))
        if not response:
            return
        if not response['data']:
            print("No files added")
            return
        for file in response['data']:
            print(file)